/*
 * audio.c
 *
 * Sun /dev/audio driver
 *
 */
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/fcntl.h>
#include <linux/sched.h>
#include "soundinit.h"
#include "soundcont.h"
#include "voice.h"

#include <asm/segment.h>

static struct wait_queue *open_wait=NULL;
static int use=0;
static int waiting=0;

static int audio_write(struct inode *inode, struct file *filp, const char *buf, int nr)
{
	int retval=0,c;
	struct voice_driver *vd=(struct voice_driver *)filp->private_data;
	const char *b = buf;
	struct wait_queue wait;

	wait.task = current;
	wait.next = NULL;

	add_wait_queue(&vd->vd_wait, &wait);
	while(1) {
		current->state = TASK_INTERRUPTIBLE;
		if(current->signal & ~current->blocked) {
			retval = -ERESTARTSYS;
			break;
		}

		c = vd->vd_write(vd,b,nr);
		b += c;
		nr -= c;
		if(nr<=0)
			break;

		if (filp->f_flags & O_NONBLOCK) {
			retval = -EAGAIN;
			break;
		}
		schedule();
	}
	current->state = TASK_RUNNING;
	remove_wait_queue(&vd->vd_wait, &wait);
	return (b - buf) ? b - buf : retval;
}

static int
audio_open(struct inode *inode, struct file *filp)
{
	int voice;
	struct voice_driver *driver;
	struct wait_queue wait;
	wait.task = current;
	wait.next = NULL;

	use++;
	if(use!=1)
	{
		use--;
		if(filp->f_flags & O_NONBLOCK)
			return -EBUSY;
		add_wait_queue(&open_wait, &wait);
		waiting++;
		while(use)
		{
			current->state = TASK_INTERRUPTIBLE;
			schedule();
		}
		waiting--;
		remove_wait_queue(&open_wait, &wait);
		use++;
	}

	if((voice=voice_allocate(VOICE_SAMPLE,0))<0)
		return voice;

	driver = (struct voice_driver *)voice;

	driver->vd_convtable	=table_linsigned;
	driver->vd_bits		=8;			/* signed 8 bit */
	filp->private_data=(void *)driver;
	voice_install(driver,0);
	return 0;
}

static void
audio_release(struct inode *inode, struct file *filp)
{
	voice_release((struct voice_driver *)filp->private_data);
	use--;
	if(waiting)
		wake_up_interruptible(&open_wait);
}

static int
audio_ioctl(struct inode * inode, struct file * file,
			     unsigned int cmd, unsigned long arg)
{/* Not yet implemented */
	return -EINVAL;
}

static struct file_operations audio_fops =
{
/* 	audio_lseek, */NULL,
/* 	audio_read, */NULL,
	audio_write,
	NULL,
/* 	audio_select, */NULL,
	audio_ioctl,
	NULL,
	audio_open,
	audio_release,
	NULL,
	NULL,
	NULL,
	NULL
};

void audio_init(void)
{
  if(register_sounddev(4,"sunaudio", &audio_fops, NULL))
    printk(KERN_ERR "Sound: can't register sun audio device\n");
}
