/*
 * linux/arch/arm/drivers/sound/soundinit.c
 *
 * Copyright (C) 1994, 1995, 1996 Russell King
 *
 * Initialise the sound system
 *
 * * Fill buffers with zero
 * * Set up stereo image registers
 * * Set up Sstart and SendN.
 * * Mark SendC for update
 *
 */

#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/signal.h>

#include <asm/irq-no.h>
#include <asm/irq.h>

#include "soundinit.h"
#include "soundirq.h"
#include "soundcont.h"

extern unsigned char sound_buffer0[],sound_buffer1[];
extern int sound_vidcsampleperiod, sound_lognchannels;
extern int sound_buffer;
extern void audio_init(void);
extern void arcaudio_init(void);

struct sound_ops snddevs[MAX_OPS];

int sound_setup(void)
{
	return 0;
}

int
register_sounddev(unsigned int minor, const char *name,
	struct file_operations *fops, void *data)
{
	if(minor >= MAX_OPS)
		return -EINVAL;
	if(snddevs[minor].name)
		return -EBUSY;
	snddevs[minor].name = name;
	snddevs[minor].fops = fops;
	snddevs[minor].data = data;
	return 0;
}

int
unregister_sounddev(unsigned int minor, const char *name)
{
	if(minor >= MAX_OPS || !snddevs[minor].name || strcmp(snddevs[minor].name,name))
		return -EINVAL;
	snddevs[minor].name = NULL;
	snddevs[minor].fops = NULL;
	return 0;
}

static int
sound_open(struct inode *inode, struct file *filp)
{
	int dev = MINOR(inode->i_rdev);
	if(dev >= MAX_OPS || !snddevs[dev].name)
		return -ENODEV;
	filp->f_op = snddevs[dev].fops;
	if(filp->f_op->open)
		return filp->f_op->open(inode,filp);
	return 0;
}

static struct file_operations sound_fops =
{
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	sound_open,
	NULL,
};

void sound_init(void)
{
	int i;

	printk (KERN_INFO "Sound: Development sound version 0.02\n");

	sound_toupdate = 0;

	for(i=0; i<SOUND_SNDBUFSIZE; i++)
		sound_buffer0[i]=sound_buffer1[i]=0;

	for(i=0; i<N_CHANNELS; i++)
	{
		sound_stereoimage[i]=SOUND_CENTRE;
		sound_voice[i]=&sound_quiet_voice;
		sound_allocated[i]=0;
	}

	/* RiscOS defaults */

	if(sound_configure(48, /* SOUND_DEFAULTCHANNELS */1, NULL, NULL)<0)
		printk(KERN_ERR "sound_configure: error\n");

	sound_samplelength=SOUND_DEFAULTLENGTH;
	sound_buffer=0;
	sound_toupdate|=SOUND_SNDFREQ|SOUND_IMAGE;

	sound_volume=-1;
	sound_vol(127);

	if(request_irq(IRQ_SOUNDCHANGE, sound_irq, SA_INTERRUPT, "sound", NULL)) {
		printk(KERN_ERR "Sound: could not claim irq %d\n",IRQ_SOUNDCHANGE);
		return;
	}

	register_chrdev(SOUND_MAJOR, "sound", &sound_fops);

	audio_init();
	arcaudio_init();
}
