/*
 * linux/arch/arm/drivers/sound/voice_samp.c
 *
 * Copyright (C) 1994, 1995, 1996 Russell King
 *
 * Sample voice driver.
 *
 * This voice driver takes is sound data from the data being written to it.
 */

#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/malloc.h>
#include "soundinit.h"
#include "soundirq.h"
#include "voice.h"

#include <asm/segment.h>

#ifndef MIN
#define MIN(a,b)	((a) < (b) ? (a) : (b))
#endif

static int voice_sample_write(struct voice_driver *vd, const char *b, int count)
{
	int c, total =0;
	unsigned long flags;

	save_flags(flags);
	while(1) {
		cli();
		c = MIN(count, MIN(vd->vd_size - vd->vd_cnt - 1,
				   vd->vd_size - vd->vd_head));
		if(c<=0)
			break;

		memcpy_fromfs(vd->vd_buf + vd->vd_head, b, c);
		vd->vd_head = (vd->vd_head + c) & (vd->vd_size-1);
		vd->vd_cnt += c;
		restore_flags(flags);
		b += c;
		count -= c;
		total += c;
	}
	restore_flags(flags);
	return total;
}

static int voice_sample_alloc(int data)
{
	struct voice_driver *vd;
	extern struct voice_driver voice_sample;

	vd = (struct voice_driver *)kmalloc(sizeof(struct voice_driver), GFP_KERNEL);
	if(!vd)
		return -ENOMEM;

	memcpy(vd, &voice_sample, sizeof(struct voice_driver));

	vd->vd_buf = (char *)kmalloc(vd->vd_size, GFP_KERNEL);
	return (int)vd;
}

static int voice_sample_install(struct voice_driver *vd, struct voice *vdrv)
{
	extern void voice_driver_fill(char *, char *, char);
	
	vdrv->fill	= voice_driver_fill;
	vdrv->gateon	= NULL;
	vdrv->gateoff	= NULL;
	vdrv->data[0]	= (int)vd;
	
	return 0;
}

const struct voice_driver voice_sample =
{
	NULL,			/* Buffer */
	0,			/* Count */
	0,			/* Tail */
	127,			/* Volume */
	NULL,			/* Conversion table */
	4096,			/* Buffer size */
	8,			/* Bits/sample */
	NULL,			/* Wait queue */
	0,			/* Head */
	-1,			/* Channel */
	voice_sample_write,	/* Write sample data */
	voice_sample_alloc,	/* Allocate new voice */
	voice_sample_install,	/* Install voice driver */
	NULL,			/* Remove voice driver (default) */
	NULL			/* Change channel volume */
};
